/*
 * productionMode.cpp
 *
 *  Created on: Jan 24, 2018
 *      Author: wn
 */


#include "defines.h"

#define MQTT_MAX_PACKET_SIZE 256


#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>

#include "configuration.h"



typedef enum { NOT_PRESSED, PRESSED, LONG_PRESSED } tPressedState;


WiFiClientSecure espClient;
PubSubClient client(espClient);

extern uint32_t startTime;


void setup_wifi() {
	delay(10);
	WiFi.mode(WIFI_STA);

	// We start by connecting to a WiFi network
#ifdef DEBUG
	Serial.println();
	Serial.print("Connecting to ");
	Serial.println(configBlock.wifiSsid);
#endif

	WiFi.begin(configBlock.wifiSsid, configBlock.wifiKey);

	while (WiFi.status() != WL_CONNECTED) {
		delay(50);
#ifdef DEBUG
		Serial.print(".");
#endif
	}
	Serial.println("!");

#ifdef DEBUG
	Serial.println("");
	Serial.println("WiFi connected");
	Serial.println("IP address: ");
	Serial.println(WiFi.localIP());
#endif
}



void reconnect() {
	// Loop until we're reconnected
	while (!client.connected()) {
#ifdef DEBUG
		Serial.print("Attempting MQTT connection...");
#endif
		// Attempt to connect
		//char clientId[128];
		//snprintf(clientId, 127, "esp%s", WiFi.macAddress().c_str());
		if (client.connect(configBlock.mqttClientId, configBlock.mqttUser, configBlock.mqttPass)) {
#ifdef DEBUG
			Serial.println("connected");
#endif
			// Once connected, publish an announcement...
			client.publish(configBlock.mqttDebugTopic, "hello world");
		} else {
#ifdef DEBUG
			Serial.print("failed, rc=");
			Serial.print(client.state());
			Serial.println(" try again in 5 seconds");
#endif
			// Wait 5 seconds before retrying
			delay(5000);
		}
	}
}


void mqtt_connect() {
	if (!client.connected()) {
		reconnect();
	}
	client.loop();
}


void setupProduction() {
	setup_wifi();
	client.setServer(configBlock.mqttBroker, configBlock.mqttPort);
	pinMode(A0, INPUT);
}


void loopProduction() {
	static tPressedState state = NOT_PRESSED;
	static uint32_t lastMillis = 0;
	mqtt_connect();


	// http://www.elektronik-labor.de/AVR/Touch.html
	// Phase 1
	pinMode(TOUCH_PIN_HIGH, INPUT);
	pinMode(TOUCH_PIN_LOW, INPUT);

	// Phase 2
	digitalWrite(TOUCH_PIN_HIGH, HIGH);
	pinMode(TOUCH_PIN_HIGH, OUTPUT);

	// Phase 3
	pinMode(TOUCH_PIN_HIGH, INPUT);

	// Phase 4
	digitalWrite(TOUCH_PIN_LOW, LOW);
	pinMode(TOUCH_PIN_LOW, OUTPUT);


	uint32_t currentMillis = millis();

	if (currentMillis - lastMillis > configBlock.period) {
		lastMillis = currentMillis;

		int v = analogRead(A0);

		if (configBlock.debugMode) {
			char payload[128];
			snprintf(payload, 127, "%ld", v);
			client.publish(configBlock.mqttDebugTopic, payload);

#ifdef DEBUG
			Serial.println(v);
#endif
		}

		if (v > configBlock.threshold) {
#ifdef DEBUG
			Serial.println(v);
#endif

			// discharge
			digitalWrite(TOUCH_PIN_HIGH, LOW);
			pinMode(TOUCH_PIN_HIGH, OUTPUT);
			digitalWrite(TOUCH_PIN_LOW, LOW);
			pinMode(TOUCH_PIN_LOW, OUTPUT);

			switch (state) {
			case NOT_PRESSED:
				state = PRESSED;
				break;
			case PRESSED:
				state = LONG_PRESSED;
				client.publish(configBlock.mqttTopic, "LONG_BEGIN");
				break;
			case LONG_PRESSED:
				state = LONG_PRESSED;
				client.publish(configBlock.mqttTopic, "LONG_CONT");
				break;
			}
		} else {
			switch (state) {
			case NOT_PRESSED:
				break;
			case PRESSED:
				client.publish(configBlock.mqttTopic, "SHORT");
				break;
			case LONG_PRESSED:
				client.publish(configBlock.mqttTopic, "LONG_END");
				break;
			}
			state = NOT_PRESSED;
		}

	}


}




